/*----------------------------------------------------------------------------
 *      U S B  -  K e r n e l
 *----------------------------------------------------------------------------
 *      Name:    cdcuser.c
 *      Purpose: USB Communication Device Class User module 
 *      Version: V1.10
 *----------------------------------------------------------------------------
*      This software is supplied "AS IS" without any warranties, express,
 *      implied or statutory, including but not limited to the implied
 *      warranties of fitness for purpose, satisfactory quality and
 *      noninfringement. Keil extends you a royalty-free right to reproduce
 *      and distribute executable files created using this software for use
 *      on NXP Semiconductors LPC microcontroller devices only. Nothing else 
 *      gives you the right to use this software.
 *
 * Copyright (c) 2009 Keil - An ARM Company. All rights reserved.
 *---------------------------------------------------------------------------*/

#include "../inc/type.h"
#include "../inc/usb.h"
#include "../inc/usbhw.h"
#include "../inc/usbcfg.h"
#include "../inc/usbcore.h"
#include "../inc/cdc.h"
#include "../inc/cdcuser.h"
#include "../../library/libDNCLABS.h"

CDC_BUF_T  CDC_OutBuf;                                 // buffer for all CDC Out data
CDC_BUF_T  CDC_InBuf;                                 // buffer for all CDC Out data

unsigned char BulkBufIn  [USB_CDC_BUFSIZE];            // Buffer to store USB IN  packet
unsigned char BulkBufOut [USB_CDC_BUFSIZE];            // Buffer to store USB OUT packet
unsigned char NotificationBuf [10];

CDC_LINE_CODING CDC_LineCoding  = {9600, 0, 0, 8};
unsigned short  CDC_SerialState = 0x0000;
unsigned short  CDC_DepInEmpty  = 1;                   // Data IN EP is empty

/*----------------------------------------------------------------------------
  CDC Initialisation
  Initializes the data structures and serial port
  Parameters:   None 
  Return Value: None
 *---------------------------------------------------------------------------*/
void CDC_Init (void) {
	CDC_DepInEmpty  = 1;
	CDC_SerialState = CDC_GetSerialState();
	CDC_BUF_RESET(CDC_OutBuf);
  
  	USB_Init();                               	// USB Initialization
	USB_Connect(TRUE);                        	// USB Connect
	while (!USB_Configuration) ;              	// wait until USB is configured
}

void USB_Putchar(unsigned char data) {
	unsigned char In_buf[1];
    while (!CDC_DepInEmpty) {}
	if(data == '\n'){
		In_buf[0] = '\r';
		In_buf[1] = '\n';
		CDC_DepInEmpty = 0;
		USB_WriteEP (CDC_DEP_IN, &In_buf[0], 2);
	}else{
		In_buf[0] = data;
		CDC_DepInEmpty = 0;
		USB_WriteEP (CDC_DEP_IN, &In_buf[0], 1);
	}
}


void USB_Putstring(char *string)
{
	volatile int i;
	for(i=0;i<80;i++){
		if(string[i] == 0){
			return;
		}else{
			USB_Putchar(string[i]);
		}
	}
}

//----------------------------------------------------------------------------
unsigned char USB_Getchar(void) {
	unsigned char result;
	while(1){
		while(!CDC_BUF_EMPTY(CDC_OutBuf)){
			result = CDC_BUF_RD(CDC_OutBuf);
			return result;
		};
	};
}

//----------------------------------------------------------------------------
unsigned char USB_Ready(void) {
	return CDC_BUF_COUNT(CDC_OutBuf);
}

/*----------------------------------------------------------------------------
  read data from
 *---------------------------------------------------------------------------*/
int CDC_RdInBuf (char *buffer, const int *length) {
	volatile int bytesToRead;
	bytesToRead = *length;
    while (bytesToRead--) {
      *buffer++ = CDC_BUF_RD(CDC_InBuf);
    }
    return (*length);
}

/*----------------------------------------------------------------------------
  write data to
 *---------------------------------------------------------------------------*/
int CDC_WrInBuf (const char *buffer, int *length) {
  volatile int bytesToWrite;//, bytesWritten;
  bytesToWrite = *length;
  if(CDC_BUF_COUNT(CDC_InBuf)>60){return 0;}
  while (bytesToWrite) {
      CDC_BUF_WR(CDC_InBuf, *buffer++);           // Copy Data to buffer
      bytesToWrite--;
  }
  return (*length);
}

/*----------------------------------------------------------------------------
  check if character(s) are available at CDC_OutBuf
 *---------------------------------------------------------------------------*/
int CDC_InBufAvailChar (int *availChar) {

  *availChar = CDC_BUF_COUNT(CDC_InBuf);

  return (0);
}
/* end Buffer handling */

/*----------------------------------------------------------------------------
  read data from CDC_OutBuf
 *---------------------------------------------------------------------------*/
int CDC_RdOutBuf (char *buffer, const int *length) {
	volatile int bytesToRead;
	bytesToRead = *length;
    while (bytesToRead--) {
      *buffer++ = CDC_BUF_RD(CDC_OutBuf);
    }
    return (*length);
}

/*----------------------------------------------------------------------------
  write data to CDC_OutBuf
 *---------------------------------------------------------------------------*/
int CDC_WrOutBuf (const char *buffer, int *length) {
  volatile int bytesToWrite;//, bytesWritten;

  bytesToWrite = *length;

  while (bytesToWrite) {
      CDC_BUF_WR(CDC_OutBuf, *buffer++);           // Copy Data to buffer  
      bytesToWrite--;
  }     
  return (*length);
}

/*----------------------------------------------------------------------------
  check if character(s) are available at CDC_OutBuf
 *---------------------------------------------------------------------------*/
int CDC_OutBufAvailChar (int *availChar) {

  *availChar = CDC_BUF_COUNT(CDC_OutBuf);

  return (0);
}
/* end Buffer handling */

/*----------------------------------------------------------------------------
  CDC SetLineCoding Request Callback
  Called automatically on CDC SET_LINE_CODING Request
  Parameters:   none                    (global SetupPacket and EP0Buf)
  Return Value: TRUE - Success, FALSE - Error
 *---------------------------------------------------------------------------*/
uint32_t CDC_SetLineCoding (void) {

  CDC_LineCoding.dwDTERate   =   (EP0Buf[0] <<  0)
                               | (EP0Buf[1] <<  8)
                               | (EP0Buf[2] << 16)
                               | (EP0Buf[3] << 24); 
  CDC_LineCoding.bCharFormat =  EP0Buf[4];
  CDC_LineCoding.bParityType =  EP0Buf[5];
  CDC_LineCoding.bDataBits   =  EP0Buf[6];

  return (TRUE);
}


/*----------------------------------------------------------------------------
  CDC GetLineCoding Request Callback
  Called automatically on CDC GET_LINE_CODING Request
  Parameters:   None                         (global SetupPacket and EP0Buf)
  Return Value: TRUE - Success, FALSE - Error
 *---------------------------------------------------------------------------*/
uint32_t CDC_GetLineCoding (void) {

  EP0Buf[0] = (CDC_LineCoding.dwDTERate >>  0) & 0xFF;
  EP0Buf[1] = (CDC_LineCoding.dwDTERate >>  8) & 0xFF;
  EP0Buf[2] = (CDC_LineCoding.dwDTERate >> 16) & 0xFF;
  EP0Buf[3] = (CDC_LineCoding.dwDTERate >> 24) & 0xFF;
  EP0Buf[4] =  CDC_LineCoding.bCharFormat;
  EP0Buf[5] =  CDC_LineCoding.bParityType;
  EP0Buf[6] =  CDC_LineCoding.bDataBits;

  return (TRUE);
}

/*----------------------------------------------------------------------------
  CDC_BulkIn call on DataIn Request
  Parameters:   none
  Return Value: none
 *---------------------------------------------------------------------------*/
void CDC_BulkIn(void) {
  int numBytesRead;
  int numBytesAvail;
  CDC_InBufAvailChar (&numBytesAvail);
  numBytesRead = CDC_WrInBuf((char *)&BulkBufIn[0], &numBytesAvail);

  // send over USB
  if (numBytesRead > 0) {
	USB_WriteEP (CDC_DEP_IN, &BulkBufIn[0], numBytesRead);
  }
  else {
    CDC_DepInEmpty = 1;
  }
}


/*----------------------------------------------------------------------------
  CDC_BulkOut call on DataOut Request
  Parameters:   none
  Return Value: none
 *---------------------------------------------------------------------------*/
void CDC_BulkOut(void) {
  int numBytesRead;

  // get data from USB into intermediate buffer
  numBytesRead = USB_ReadEP(CDC_DEP_OUT, &BulkBufOut[0]);

  // ... add code to check for overwrite

  // store data in a buffer to transmit it over serial interface
  CDC_WrOutBuf ((char *)&BulkBufOut[0], &numBytesRead);

}


/*----------------------------------------------------------------------------
  Get the SERIAL_STATE as defined in usbcdc11.pdf, 6.3.5, Table 69.
  Parameters:   none
  Return Value: SerialState as defined in usbcdc11.pdf
 *---------------------------------------------------------------------------*/
unsigned short CDC_GetSerialState (void) {
  unsigned short temp=0;

  CDC_SerialState = 0;
 // ser_LineState (&temp);

  if (temp & 0x8000)  CDC_SerialState |= CDC_SERIAL_STATE_RX_CARRIER;
  if (temp & 0x2000)  CDC_SerialState |= CDC_SERIAL_STATE_TX_CARRIER;
  if (temp & 0x0010)  CDC_SerialState |= CDC_SERIAL_STATE_BREAK;
  if (temp & 0x4000)  CDC_SerialState |= CDC_SERIAL_STATE_RING;
  if (temp & 0x0008)  CDC_SerialState |= CDC_SERIAL_STATE_FRAMING;
  if (temp & 0x0004)  CDC_SerialState |= CDC_SERIAL_STATE_PARITY;
  if (temp & 0x0002)  CDC_SerialState |= CDC_SERIAL_STATE_OVERRUN;

  return (CDC_SerialState);
}


/*----------------------------------------------------------------------------
  Send the SERIAL_STATE notification as defined in usbcdc11.pdf, 6.3.5.
 *---------------------------------------------------------------------------*/
void CDC_NotificationIn (void) {

  NotificationBuf[0] = 0xA1;                           // bmRequestType
  NotificationBuf[1] = CDC_NOTIFICATION_SERIAL_STATE;  // bNotification (SERIAL_STATE)
  NotificationBuf[2] = 0x00;                           // wValue
  NotificationBuf[3] = 0x00;
  NotificationBuf[4] = 0x00;                           // wIndex (Interface #, LSB first)
  NotificationBuf[5] = 0x00;
  NotificationBuf[6] = 0x02;                           // wLength (Data length = 2 bytes, LSB first)
  NotificationBuf[7] = 0x00; 
  NotificationBuf[8] = (CDC_SerialState >>  0) & 0xFF; // UART State Bitmap (16bits, LSB first)
  NotificationBuf[9] = (CDC_SerialState >>  8) & 0xFF;

  USB_WriteEP (CDC_CEP_IN, &NotificationBuf[0], 10);   // send notification
}


